module EventSourcery
  module DynamoDB
    class Tracker
      def initialize(connection, table_name:, obtain_processor_lock: true)
        @connection = connection
        @table_name = table_name.to_s
        @obtain_processor_lock = obtain_processor_lock
      end

      def setup(processor_name = nil)
        create_table_if_not_exists
        create_track_entry_if_not_exists(processor_name)
      end

      def processed_event(processor_name, event_id)
        update_track_entry(processor_name, event_id)
      end

      def reset_last_processed_event_id(processor_name)
        update_track_entry(processor_name, 0)
      end

      def last_processed_event_id(processor_name)
        track_entry = get_track_entry(processor_name)
        track_entry.fetch('last_processed_event_id').to_i
      end

      def tracked_processors
      end

      private

      attr_reader :table_name, :connection

      def create_table_if_not_exists
        tables = connection.list_tables.to_h
        unless tables[:table_names].include?(table_name)
          ::EventSourcery::DynamoDB::Schema.create_projector_tracker(db: connection, table_name: table_name)
        end
      end

      def create_track_entry_if_not_exists(processor_name)
        track_entry = get_track_entry(processor_name)
        if track_entry.nil?
          update_track_entry(processor_name, 0)
        end
      end

      def get_track_entry(processor_name)
        connection.get_item({
          table_name: table_name,
          key: { 'name' => processor_name }
        }).item
      end

      def update_track_entry(processor_name, event_id)
        connection.update_item({
          table_name: table_name,
          key: { 'name' => processor_name },
          attribute_updates: {
            'last_processed_event_id' => { value: event_id }
          }
        })
      end
    end
  end
end
